
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Squared Tower materials
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SST_RED_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_SST_BLUE_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_SST_BROWN_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_SST_GREY_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_SST_GREY2_ROOF_MATERIAL")

-- Register Small Squared Tower prefab nodes
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", StoneGateSetPrefix .. "_SST_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.05 }
	    }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_SST_TOP",
	StoneGateSetPrefix .. "_SST_TILING1",
	StoneGateSetPrefix .. "_SST_TILING2",
	StoneGateSetPrefix .. "_SST_TILING1P",
	StoneGateSetPrefix .. "_SST_TILING2P"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerSmallSquaredTower(_option)
    -- Register Small Squared Tower prefab nodes
    StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PREFAB")

    -- Register the stone gate set small squared tower root part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _SstBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	},
        VillagerCapacity = 3,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
    })

    -- Register grass clearing components
    StoneGateV5:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones   
    StoneGateV5:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 4 }
   })
end

registerSmallSquaredTower("") -- with stone ground
registerSmallSquaredTower("P") -- with wooden ground

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", { "PLATFORM" })
 
function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 3 }
		}
	}
    })
end

registerRoofPart( "RoofA", "ROOF_A" )
registerRoofPart( "RoofB", "ROOF_B" )
registerRoofPart( "RoofC", "ROOF_C" )
registerRoofPart( "RoofD", "ROOF_D" )
registerRoofPart( "RoofE", "ROOF_E" )
registerRoofPart( "RoofF", "ROOF_F" )

---- Ramparts...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/ramparts/rampartSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGateV5:registerAssetProcessor("models/ramparts/bridge.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType)
    StoneGateV5:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType, -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 }
    })
end

registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartA", "RAMPART_A", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartB", "RAMPART_B", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartC", "RAMPART_C", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartD", "RAMPART_D", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "RampartE", "RAMPART_E", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "StairA", "STAIR_A", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "StairB", "STAIR_B", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "StairC", "STAIR_C", "MAJOR" )
registerRampartPart( "ramparts/rampartSmallSquaredTower", "StairD", "STAIR_D", "MAJOR" )
registerRampartPart( "ramparts/bridge", "Bridge1", "BRIDGE1", "MINOR" )
registerRampartPart( "ramparts/bridge", "Bridge1P", "BRIDGE1P", "MINOR" )
registerRampartPart( "ramparts/bridge", "HalfArch1", "HALF_ARCH1", "MINOR" )
registerRampartPart( "ramparts/bridge", "HalfArch2", "HALF_ARCH2", "MINOR" )
